# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/10_logr.ipynb.

# %% auto 0
__all__ = ['ArgsType', 'SysExcInfoType', 'FormatStyle']

# %% ../nbs/10_logr.ipynb 6
import inspect

# %% ../nbs/10_logr.ipynb 8
from types import TracebackType
from typing import Any, Mapping, Literal, TypeAlias

# %% ../nbs/10_logr.ipynb 10
#| export


# %% ../nbs/10_logr.ipynb 12
#| export


# %% ../nbs/10_logr.ipynb 14
#| export


# %% ../nbs/10_logr.ipynb 16
ArgsType: TypeAlias = type[tuple[object, ...]] | type[Mapping[str, object]]

SysExcInfoType: TypeAlias = \
    type[tuple[type[BaseException], BaseException, TracebackType | None]] | \
    type[tuple[None, None, None]]

FormatStyle: TypeAlias = Literal['%', '{', '$']



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_enum.ipynb.

# %% auto 0
__all__ = ['Intum', 'Strum', 'Inhum']

# %% ../nbs/05_enum.ipynb 6
from enum import IntEnum, StrEnum

# %% ../nbs/05_enum.ipynb 8
from typing import (
    TypeAlias, Literal, Union, List, Set,
    Iterable, Tuple, ForwardRef
)

# %% ../nbs/05_enum.ipynb 10
#| export


# %% ../nbs/05_enum.ipynb 12
#| export


# %% ../nbs/05_enum.ipynb 14
#| export


# %% ../nbs/05_enum.ipynb 16
Intum: TypeAlias = Union[IntEnum, 'LitIntEnum', 'Intum']
Strum: TypeAlias = Union[StrEnum, 'LitStrEnum', 'Strum', 'LiteralEnum']

Inhum = ForwardRef('Inhum', module='inhum', is_class=True)



__version__ = "0.0.46"
# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00__init__.ipynb.

# %% auto 0
__all__ = []

# %% ../nbs/00__init__.ipynb 3
from itertools import chain
from atyp import (
    # atyp as amod,
    bool as bmod, nums, strs, path, enum, byte, type as tmod, imps, insp, logr, clsv, 
    typq, tgrd, iter as imod, list as lmod, sets as smod, rich, tqdm, plot, typr, 
    # data, 
    biol, tens, imgs, cstm, npty, pdty, spty
)

# from atyp.atyp import *
from .bool import *
from .nums import *
from .strs import *
from .path import *
from .enum import *
from .byte import *
from .type import *
from .imps import *
from .insp import *
from .logr import *
from .clsv import *
from .typq import *
from .tgrd import *
from .iter import *
from .list import *
from .sets import *
from .rich import *
from .tqdm import *
from .plot import *
from .typr import *
# from atyp.data import *
from .biol import *
from .tens import *
from .imgs import *
from .cstm import *
from .npty import *
from .pdty import *
from .spty import *

# %% ../nbs/00__init__.ipynb 4
__all__ = sorted(set(chain(*[
    mod.__all__ for mod in (
        # amod,
        bmod, nums, strs, path, enum, byte, tmod, imps, insp, logr, clsv, 
        typq, tgrd, imod, lmod, smod, rich, tqdm, plot, typr, 
        # data, 
        biol, tens, imgs, cstm,
        npty, pdty, spty
    )
])))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/32_tgrd.ipynb.

# %% auto 0
__all__ = ['T', 'P', 'TypeLike', 'GuardFunc', 'StrGuard']

# %% ../nbs/32_tgrd.ipynb 6
#| export


# %% ../nbs/32_tgrd.ipynb 8
from typing import Type, Union, Callable, TypeAlias, TypeGuard, TypeVar, ParamSpec
T = TypeVar('T')
P = ParamSpec('P')

# %% ../nbs/32_tgrd.ipynb 10
#| export


# %% ../nbs/32_tgrd.ipynb 12
#| export


# %% ../nbs/32_tgrd.ipynb 14
#| export


# %% ../nbs/32_tgrd.ipynb 16
TypeLike:  TypeAlias = Union[Type, TypeAlias, Type[T]]
GuardFunc: TypeAlias = Callable[P, TypeGuard[TypeLike]]

# %% ../nbs/32_tgrd.ipynb 17
StrGuard: TypeAlias = Callable[[str], TypeGuard[str]]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/07_type.ipynb.

# %% auto 0
__all__ = ['Literals', 'TypingLike', 'GuardLike', 'Module']

# %% ../nbs/07_type.ipynb 6
#| export


# %% ../nbs/07_type.ipynb 8
import types
from typing import (
    TypeAlias, Literal, Union, List, Set,
    Iterable, Tuple, ForwardRef, TypeVar, TypeGuard
)

# %% ../nbs/07_type.ipynb 10
#| export


# %% ../nbs/07_type.ipynb 12
#| export


# %% ../nbs/07_type.ipynb 14
#| export


# %% ../nbs/07_type.ipynb 17
Literals = TypeVar('Literals')
TypingLike: TypeAlias = Union['TTyp', 'TType', 'TypingUnit', 'Typing']
GuardLike: TypeAlias = Union['TTypGuard', 'TTypeGuard']

# %% ../nbs/07_type.ipynb 19
Module: TypeAlias = types.ModuleType



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/atyp',
                'doc_host': 'https://dsm-72.github.io',
                'git_url': 'https://github.com/dsm-72/atyp',
                'lib_path': 'atyp'},
  'syms': { 'atyp.biol': {},
            'atyp.bool': {'atyp.bool.Bool01': ('bool.html#bool01', 'atyp/bool.py')},
            'atyp.byte': {},
            'atyp.clsv': {},
            'atyp.cstm': {},
            'atyp.enum': {},
            'atyp.imgs': {},
            'atyp.imps': {},
            'atyp.insp': {},
            'atyp.iter': {},
            'atyp.list': {},
            'atyp.logr': {},
            'atyp.npty': {},
            'atyp.nums': {},
            'atyp.path': {},
            'atyp.pdty': { 'atyp.pdty.BoolIndex': ('pdty.html#boolindex', 'atyp/pdty.py'),
                           'atyp.pdty.IndexLike': ('pdty.html#indexlike', 'atyp/pdty.py'),
                           'atyp.pdty.SeriesLike': ('pdty.html#serieslike', 'atyp/pdty.py')},
            'atyp.plot': {},
            'atyp.rich': {},
            'atyp.sets': {},
            'atyp.spty': {},
            'atyp.strs': { 'atyp.strs.NotStrBytes': ('strs.html#notstrbytes', 'atyp/strs.py'),
                           'atyp.strs.StrBytes': ('strs.html#strbytes', 'atyp/strs.py')},
            'atyp.tens': {},
            'atyp.tgrd': {},
            'atyp.tqdm': {},
            'atyp.type': {},
            'atyp.typq': {},
            'atyp.typr': {}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/06_byte.ipynb.

# %% auto 0
__all__ = ['ByteLike']

# %% ../nbs/06_byte.ipynb 6
#| export


# %% ../nbs/06_byte.ipynb 8
from typing import (
    TypeAlias, Literal, Union, List, Set,
    Iterable, Tuple, ForwardRef
)

# %% ../nbs/06_byte.ipynb 10
#| export


# %% ../nbs/06_byte.ipynb 12
#| export


# %% ../nbs/06_byte.ipynb 14
#| export


# %% ../nbs/06_byte.ipynb 16
ByteLike: TypeAlias = Union[bytes, 'BytesUnit', 'Bytes']



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/50_rich.ipynb.

# %% auto 0
__all__ = ['RichTree', 'RichText', 'RichCons', 'RichProg']

# %% ../nbs/50_rich.ipynb 6
import inspect

# %% ../nbs/50_rich.ipynb 8
import types
from typing import (
    Type, TypeAlias, 
    ClassVar, Any, Optional, Callable, Iterable, ForwardRef
)
from numbers import Number

# %% ../nbs/50_rich.ipynb 10
try:
    from rich.tree import Tree
    from rich.text import Text
    from rich.console import Console
    from rich.progress import Progress
    tree = Tree
    text = Text
    cons = Console
    prog = Progress
    
except ImportError:
    tree = ForwardRef('rich.tree.Tree', module='rich.tree', is_class=True)
    text = ForwardRef('rich.text.Text', module='rich.text', is_class=True)
    cons = ForwardRef('rich.console.Console', module='rich.console', is_class=True)
    prog = ForwardRef('rich.progress.Progress', module='rich.progress', is_class=True)

# %% ../nbs/50_rich.ipynb 12
#| export


# %% ../nbs/50_rich.ipynb 14
#| export


# %% ../nbs/50_rich.ipynb 16
RichTree: TypeAlias = tree
RichText: TypeAlias = text
RichCons: TypeAlias = cons
RichProg: TypeAlias = prog



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/110_pdty.ipynb.

# %% auto 0
__all__ = ['Idx', 'Index', 'CIdx', 'CIndex', 'CatIndex', 'CategoricalIndex', 'BIdx', 'BooleanIndex', 'Series', 'DFrame',
           'DataFrame', 'DataFrames', 'IterLike', 'GroupKey', 'BoolIndex', 'IndexLike', 'SeriesLike']

# %% ../nbs/110_pdty.ipynb 6
#| export


# %% ../nbs/110_pdty.ipynb 8
from typing import TypeAlias, List, Union, Iterable, List

# %% ../nbs/110_pdty.ipynb 11
try:
    
    import pandas as pd
    index = pd.Index
    cindex = pd.CategoricalIndex
    
    dframe = pd.DataFrame
    series = pd.Series

except ImportError:
    
    # Define a placeholder type if pandas is not installed
    index = 'pd.Index'    
    cindex = 'pd.CategoricalIndex'
    
    dframe = 'pd.DataFrame'
    series = 'pd.Series'

# %% ../nbs/110_pdty.ipynb 13
from ptyp import Alias

# %% ../nbs/110_pdty.ipynb 15
from .bool import Boolish, Bool01
from .nums import Numbers
from .tens import Tensor
from .npty import ndarray

# %% ../nbs/110_pdty.ipynb 18
Idx: TypeAlias = index
Index: TypeAlias = index

CIdx: TypeAlias = cindex
CIndex: TypeAlias = cindex
CatIndex: TypeAlias = cindex
CategoricalIndex: TypeAlias = cindex

# %% ../nbs/110_pdty.ipynb 19
# NOTE: (bool | Bool01) == Union[bool, Bool01]
# @Alias(index, ndarray, list, attrs=dict(dtype=bool | Bool01))
# @Alias(index, ndarray, list, attrs=dict(dtype=Union[bool, Bool01]))
@Alias(index, ndarray, list, attrs=dict(dtype=Bool01))
class BoolIndex: ...

BIdx: TypeAlias = BoolIndex
BooleanIndex: TypeAlias = BoolIndex

# %% ../nbs/110_pdty.ipynb 22
Series: TypeAlias = series

DFrame: TypeAlias = dframe
DataFrame: TypeAlias = dframe
DataFrames: TypeAlias = List[DataFrame]

# %% ../nbs/110_pdty.ipynb 24
# IndexLike: TypeAlias = Union[Numbers, List[Boolish], Index]
@Alias(index, cindex, ndarray, BoolIndex, list, Numbers)
class IndexLike: ...

@Alias(series, ndarray, list, IndexLike)
class SeriesLike: ...

# SeriesLike: TypeAlias = Union[list, Series, ndarray, Tensor, IndexLike]
IterLike: TypeAlias = Union[Iterable, SeriesLike, IndexLike]

# %% ../nbs/110_pdty.ipynb 25
GroupKey: TypeAlias = Union[str, int, float, tuple] # key for pandas groupby



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/110_spty.ipynb.

# %% auto 0
__all__ = ['SPArray', 'SPMatrix']

# %% ../nbs/110_spty.ipynb 6
#| export


# %% ../nbs/110_spty.ipynb 8
from typing import TypeAlias

# %% ../nbs/110_spty.ipynb 11
try:
    import scipy as sp
    
    try: sparr = sp.sparse._arrays._sparray
    except AttributeError: sparr = sp.sparse.sparray

    spmtx = sp.sparse.spmatrix

except ImportError:
    # Define a placeholder type if scipy is not installed
    sparr = 'scipy.sparse._base.sparray'
    spmtx = 'scipy.sparse._matrix.spmatrix'

# %% ../nbs/110_spty.ipynb 13
#| export


# %% ../nbs/110_spty.ipynb 15
#| export


# %% ../nbs/110_spty.ipynb 18
SPArray: TypeAlias = sparr
SPMatrix: TypeAlias = spmtx



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/90_cstm.ipynb.

# %% auto 0
__all__ = ['PathFormatFunc', 'Mixin']

# %% ../nbs/90_cstm.ipynb 6
import os
from numbers import Number

# %% ../nbs/90_cstm.ipynb 8
from typing import (Type, Union, Tuple, TypeAlias, Callable)

# %% ../nbs/90_cstm.ipynb 10
#| export


# %% ../nbs/90_cstm.ipynb 12
#| export


# %% ../nbs/90_cstm.ipynb 14
from .bool import Boolish
from .nums import Numeric
from .typq import AnyQ, StrQ

# %% ../nbs/90_cstm.ipynb 16
PathFormatFunc: TypeAlias = Callable[[os.PathLike, StrQ, StrQ, StrQ], str]

# %% ../nbs/90_cstm.ipynb 18
Mixin: TypeAlias =  Union[Tuple[Type, Callable], 'MixinType']



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/50_tqdm.ipynb.

# %% auto 0
__all__ = ['TQDM']

# %% ../nbs/50_tqdm.ipynb 6
#| export


# %% ../nbs/50_tqdm.ipynb 8
from typing import (TypeAlias, ForwardRef)

# %% ../nbs/50_tqdm.ipynb 10
try: 
    from tqdm.auto import tqdm as auto    
except ImportError: 
    auto = ForwardRef('tqdm.auto.tqdm', module='tqdm.auto', is_class=True)

# %% ../nbs/50_tqdm.ipynb 12
#| export


# %% ../nbs/50_tqdm.ipynb 14
#| export


# %% ../nbs/50_tqdm.ipynb 16
TQDM: TypeAlias = auto



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/51_typr.ipynb.

# %% auto 0
__all__ = ['TyperApp']

# %% ../nbs/51_typr.ipynb 6
import inspect

# %% ../nbs/51_typr.ipynb 8
import types
from typing import (
    Type, TypeAlias, 
    ClassVar, Any, Optional, Callable, Iterable, ForwardRef
)
from numbers import Number

# %% ../nbs/51_typr.ipynb 10
try:
    from typer import Typer
    tapp = Typer    
except ImportError:
    tapp = ForwardRef('typer.Typer', module='typer', is_class=True)

# %% ../nbs/51_typr.ipynb 12
#| export


# %% ../nbs/51_typr.ipynb 14
#| export


# %% ../nbs/51_typr.ipynb 16
TyperApp: TypeAlias = tapp



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/54_tens.ipynb.

# %% auto 0
__all__ = ['dtype', 'DType', 'datatype', 'DataType', 'Tensor', 'Device', 'Layer', 'Module', 'Dataset', 'DataLoader', 'DataModule']

# %% ../nbs/54_tens.ipynb 6
import inspect
from numbers import Number

# %% ../nbs/54_tens.ipynb 8
import types
from typing import (
    Type, TypeAlias, Tuple,
    ClassVar, Any, Optional, Callable, Iterable, ForwardRef
)
from typing import TypeAlias, List, Union, Iterable, List

# %% ../nbs/54_tens.ipynb 11
try:
    import torch
    torchtype = torch.dtype
    tensor = torch.Tensor
    device = torch.device
    module = torch.nn.Module
except ImportError:
    # Define a placeholder type if torch is not installed
    tensor = 'torch.Tensor'
    device = 'torch.device'
    torchtype = 'torch.dtype'
    module = 'torch.nn.Module'

# %% ../nbs/54_tens.ipynb 12
try:
    from torch.utils.data import (
        Dataset as dataset, 
        DataLoader as dataloader
    )    
except ImportError:
    tudata = 'torch.utils.data'
    dataset = ForwardRef(f'{tudata}.Dataset', module=tudata, is_class=True)
    dataloader = ForwardRef(f'{tudata}.DataLoader', module=tudata, is_class=True)

# %% ../nbs/54_tens.ipynb 13
try:
    from pytorch_lightning import (LightningDataModule as pldm)
except ImportError:
    pl = 'pytorch_lightning'
    pldm = ForwardRef(f'{pl}.LightningDataModule', module='pytorch_lightning', is_class=True)

# %% ../nbs/54_tens.ipynb 16
#| export


# %% ../nbs/54_tens.ipynb 18
from .nums import Numbers

# %% ../nbs/54_tens.ipynb 21
dtype: TypeAlias = torchtype
DType: TypeAlias = dtype
datatype: TypeAlias = dtype
DataType: TypeAlias = dtype

# %% ../nbs/54_tens.ipynb 22
Tensor: TypeAlias = tensor
Device: TypeAlias = device

# %% ../nbs/54_tens.ipynb 23
Layer: TypeAlias = module
Module: TypeAlias = Layer

# %% ../nbs/54_tens.ipynb 25
Dataset: TypeAlias = dataset
DataLoader: TypeAlias = dataloader

# %% ../nbs/54_tens.ipynb 26
DataModule: TypeAlias = pldm



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/51_plot.ipynb.

# %% auto 0
__all__ = ['Axes', 'ColorMap', 'CBarPos']

# %% ../nbs/51_plot.ipynb 6
import inspect

# %% ../nbs/51_plot.ipynb 8
import types
from typing import (
    Type, TypeAlias, Tuple,
    ClassVar, Any, Optional, Callable, Iterable, ForwardRef
)
from numbers import Number

# %% ../nbs/51_plot.ipynb 10
try:
    import matplotlib as mpl
    from matplotlib.axes._axes import Axes

    cmap = mpl.colors.Colormap
    axes = Axes

except ImportError:
    cmap = 'matplotlib.colors.Colormap'
    axes = 'matplotlib.axes._axes.Axes'

# %% ../nbs/51_plot.ipynb 12
#| export


# %% ../nbs/51_plot.ipynb 14
#| export


# %% ../nbs/51_plot.ipynb 16
Axes: TypeAlias = axes
ColorMap: TypeAlias = cmap

# CBarPos: tuple of (left, bottom, width, height)
CBarPos: TypeAlias = Tuple[float, float, float, float]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_bool.ipynb.

# %% auto 0
__all__ = ['P', 'Boolic', 'Boolish', 'Bools', 'BoolFunc', 'BoolCall', 'BoolCalls', 'BoolFuncQ', 'BoolCallQ', 'FlagFunc',
           'FlagFuncQ', 'Bool01']

# %% ../nbs/01_bool.ipynb 6
#| export


# %% ../nbs/01_bool.ipynb 8
from typing import (Any, List, Union, Callable, Literal, TypeAlias, ParamSpec, Optional)
P = ParamSpec('P')

# %% ../nbs/01_bool.ipynb 10
try: from numpy import bool_ as npbool
except ImportError: npbool = bool

# %% ../nbs/01_bool.ipynb 12
#| export


# %% ../nbs/01_bool.ipynb 14
from ptyp import Alias

# %% ../nbs/01_bool.ipynb 16
Boolic:  TypeAlias = Union[bool, npbool]

@Alias(bool, npbool, 0, 1)
class Bool01: ...
# Bool01: TypeAlias = Union[Literal[0], Literal[1]]

Boolish: TypeAlias = Union[bool, Boolic, Bool01]

Bools: TypeAlias = List[bool]
'''A type alias for a list of booleans.''';

# %% ../nbs/01_bool.ipynb 17
BoolFunc: TypeAlias = Callable[P, Boolish]
'''A type alias for a function that returns a boolean.'''

BoolCall: TypeAlias = Union[Boolish, Callable]
'''A type alias for a boolean or a callable that returns a boolean.'''

BoolCalls: TypeAlias = List[BoolCall]
'''A type alias for a list of booleans or callables that each returns a boolean.'''

# %% ../nbs/01_bool.ipynb 18
BoolFuncQ: TypeAlias = Optional[BoolFunc]
BoolCallQ: TypeAlias = Optional[BoolCall]

# %% ../nbs/01_bool.ipynb 20
FlagFunc: TypeAlias = Callable[[Any], Boolish]
FlagFuncQ: TypeAlias = Optional[FlagFunc]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/41_list.ipynb.

# %% auto 0
__all__ = ['ListStr', 'ListNum', 'ListBool', 'Strs', 'Nums', 'Bools']

# %% ../nbs/41_list.ipynb 6
import inspect

# %% ../nbs/41_list.ipynb 8
from typing import (TypeAlias, List)

# %% ../nbs/41_list.ipynb 10
#| export


# %% ../nbs/41_list.ipynb 12
#| export


# %% ../nbs/41_list.ipynb 14
from .bool import Boolish
from .nums import Numeric

# %% ../nbs/41_list.ipynb 16
ListStr: TypeAlias = List[str]
ListNum: TypeAlias = List[Numeric]
ListBool: TypeAlias = List[Boolish]


Strs: TypeAlias = ListStr
Nums: TypeAlias = ListNum
Bools: TypeAlias = ListBool



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/30_clsv.ipynb.

# %% auto 0
__all__ = ['ClsInt', 'ClsSet', 'ClsStr', 'ClsObj', 'ClsBool', 'ClsBoolish', 'ClsCall', 'ClsType', 'ClsList', 'ClsFloat',
           'ClsNumber', 'ClsComplex']

# %% ../nbs/30_clsv.ipynb 6
import inspect

# %% ../nbs/30_clsv.ipynb 8
import types
from typing import (
    TypeAlias, Literal, Union, List, Set,
    Iterable, Tuple, ForwardRef, TypeVar, TypeGuard
)
from typing import (ClassVar, Callable)

# %% ../nbs/30_clsv.ipynb 10
#| export


# %% ../nbs/30_clsv.ipynb 12
#| export


# %% ../nbs/30_clsv.ipynb 14
from .bool import Boolish
from .nums import Numeric

# %% ../nbs/30_clsv.ipynb 16
ClsInt = ClassVar[int]
ClsSet = ClassVar[set]
ClsStr = ClassVar[str]
ClsObj = ClassVar[object]

ClsBool = ClassVar[bool]
ClsBoolish = ClassVar[Boolish]
ClsCall = ClassVar[Callable]

ClsType = ClassVar[type]
ClsList = ClassVar[list]

ClsFloat = ClassVar[float]
ClsNumber = ClassVar[Numeric]
ClsComplex = ClassVar[complex]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_strs.ipynb.

# %% auto 0
__all__ = ['StrTmpl', 'StrLike', 'Pttn', 'PttnDict', 'StrBytes', 'NotStrBytes']

# %% ../nbs/03_strs.ipynb 6
import re
from string import Template

# %% ../nbs/03_strs.ipynb 8
from typing import (TypeAlias, Union, Dict)

# %% ../nbs/03_strs.ipynb 10
#| export


# %% ../nbs/03_strs.ipynb 12
from ptyp import Alias, Not

# %% ../nbs/03_strs.ipynb 14
#| export


# %% ../nbs/03_strs.ipynb 16
StrTmpl: TypeAlias = Union[str, Template]
StrLike: TypeAlias = Union[str, 'strfix', 'fuzstr', 'nymstr', 'synset', 'brvstr', 'strfix']

# %% ../nbs/03_strs.ipynb 18
Pttn: TypeAlias = re.Pattern
PttnDict: TypeAlias = Dict[str, Pttn]

# %% ../nbs/03_strs.ipynb 19
@Alias(str, bytes)
class StrBytes: ...

@Not(str, bytes)
class NotStrBytes: ...



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/08_imps.ipynb.

# %% auto 0
__all__ = ['Loader', 'ModuleSpec']

# %% ../nbs/08_imps.ipynb 6
import importlib
from importlib.abc import Loader

# %% ../nbs/08_imps.ipynb 8
import types
from typing import (
    TypeAlias, Literal, Union, List, Set,
    Iterable, Tuple, ForwardRef, TypeVar, TypeGuard
)

# %% ../nbs/08_imps.ipynb 10
#| export


# %% ../nbs/08_imps.ipynb 12
#| export


# %% ../nbs/08_imps.ipynb 14
#| export


# %% ../nbs/08_imps.ipynb 16
Loader: TypeAlias = Loader
ModuleSpec: TypeAlias = importlib.machinery.ModuleSpec



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/31_typq.ipynb.

# %% auto 0
__all__ = ['AnyQ', 'IntQ', 'SetQ', 'StrQ', 'ObjQ', 'BoolQ', 'DictQ', 'CallQ', 'TypeQ', 'ListQ', 'IterQ', 'FloatQ', 'TupleQ',
           'NumberQ', 'ComplexQ', 'StrLikeQ', 'NumStrQ', 'PttnQ', 'PatternQ', 'LoggerQ', 'ModuleQ', 'PathLikeQ',
           'BoolFuncQ', 'BoolCallQ', 'ClsAnyQ', 'ClsIntQ', 'ClsSetQ', 'ClsStrQ', 'ClsObjQ', 'ClsBoolQ', 'ClsCallQ',
           'ClsTypeQ', 'ClsListQ', 'ClsIterQ', 'ClsFloatQ', 'ClsNumberQ', 'ClsComplexQ', 'IdxQ', 'IndexQ', 'SeriesQ',
           'DataFrameQ', 'CatQ', 'CIdxQ', 'CatIndexQ', 'BIdxQ', 'BoolIndexQ', 'IndexLikeQ', 'NPArrayQ', 'NPMatrixQ',
           'SPArrayQ', 'SPMatrixQ', 'DTypeQ', 'DeviceQ', 'TensorQ', 'DatasetQ', 'DataLoaderQ', 'DataModuleQ', 'VidCapQ',
           'ImgSizeQ']

# %% ../nbs/31_typq.ipynb 6
import logging

# %% ../nbs/31_typq.ipynb 8
from typing import (Type, TypeAlias, ClassVar, Any, Optional, Callable, Iterable,)

# %% ../nbs/31_typq.ipynb 10
#| export


# %% ../nbs/31_typq.ipynb 12
#| export


# %% ../nbs/31_typq.ipynb 14
from .bool import Boolish, BoolFunc, BoolCall
from .nums import Numeric, NumStr
from .strs import StrLike, Pttn
from .path import PathLike
from .type import Module
from .pdty import Idx, Index, Series, DataFrame
from .pdty import CatIndex, BoolIndex, IndexLike
from .npty import NPArray, NPMatrix
from .spty import SPArray, SPMatrix
from .imgs import VidCap, ImgSize
from .tens import DType, Device, Tensor, Dataset, DataLoader, DataModule

# %% ../nbs/31_typq.ipynb 17
AnyQ: TypeAlias = Optional[Any]
IntQ: TypeAlias = Optional[int]
SetQ: TypeAlias = Optional[set]
StrQ: TypeAlias = Optional[str]
ObjQ: TypeAlias = Optional[object]

BoolQ: TypeAlias = Optional[Boolish]
DictQ: TypeAlias = Optional[dict]
CallQ: TypeAlias = Optional[Callable]
TypeQ: TypeAlias = Optional[Type]
ListQ: TypeAlias = Optional[list]
IterQ: TypeAlias = Optional[Iterable]
FloatQ: TypeAlias = Optional[float]
TupleQ: TypeAlias = Optional[tuple]
NumberQ: TypeAlias = Optional[Numeric]
ComplexQ: TypeAlias = Optional[complex]

StrLikeQ: TypeAlias = Optional[StrLike]
NumStrQ: TypeAlias = Optional[NumStr]

# %% ../nbs/31_typq.ipynb 19
PttnQ: TypeAlias = Optional[Pttn]
PatternQ: TypeAlias = Optional[Pttn]

# %% ../nbs/31_typq.ipynb 21
LoggerQ: TypeAlias = Optional[logging.Logger]

# %% ../nbs/31_typq.ipynb 23
ModuleQ: TypeAlias = Optional[Module]

PathLikeQ: TypeAlias = Optional[PathLike]

# %% ../nbs/31_typq.ipynb 25
BoolFuncQ: TypeAlias = Optional[BoolFunc]

BoolCallQ: TypeAlias = Optional[BoolCall]

# %% ../nbs/31_typq.ipynb 27
ClsAnyQ = ClassVar[AnyQ]
ClsIntQ = ClassVar[IntQ]
ClsSetQ = ClassVar[SetQ]
ClsStrQ = ClassVar[StrQ]
ClsObjQ = ClassVar[ObjQ]
ClsBoolQ = ClassVar[BoolQ]
ClsCallQ = ClassVar[CallQ]
ClsTypeQ = ClassVar[TypeQ]
ClsListQ = ClassVar[ListQ]
ClsIterQ = ClassVar[IterQ]
ClsFloatQ = ClassVar[FloatQ]
ClsNumberQ = ClassVar[NumberQ]
ClsComplexQ = ClassVar[ComplexQ]

StrLikeQ: TypeAlias = Optional[StrLike]

# %% ../nbs/31_typq.ipynb 29
IdxQ: TypeAlias = Optional[Idx]
IndexQ: TypeAlias = Optional[Index]
SeriesQ: TypeAlias = Optional[Series]
DataFrameQ: TypeAlias = Optional[DataFrame]

# %% ../nbs/31_typq.ipynb 30
CatQ: TypeAlias = Optional[CatIndex]
CIdxQ: TypeAlias = Optional[CatIndex]
CatIndexQ: TypeAlias = Optional[CatIndex]


BIdxQ: TypeAlias = Optional[BoolIndex]
BoolIndexQ: TypeAlias = Optional[BoolIndex]
IndexLikeQ: TypeAlias = Optional[IndexLike]

# %% ../nbs/31_typq.ipynb 31
NPArrayQ: TypeAlias = Optional[NPArray]
NPMatrixQ: TypeAlias = Optional[NPMatrix]

SPArrayQ: TypeAlias = Optional[SPArray]
SPMatrixQ: TypeAlias = Optional[SPMatrix]

# %% ../nbs/31_typq.ipynb 33
DTypeQ: TypeAlias = Optional[DType]
DeviceQ: TypeAlias = Optional[Device]
TensorQ: TypeAlias = Optional[Tensor]

# %% ../nbs/31_typq.ipynb 34
DatasetQ: TypeAlias = Optional[Dataset]
DataLoaderQ: TypeAlias = Optional[DataLoader]
DataModuleQ: TypeAlias = Optional[DataModule]

# %% ../nbs/31_typq.ipynb 36
VidCapQ: TypeAlias = Optional[VidCap]
ImgSizeQ: TypeAlias = Optional[ImgSize]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/09_insp.ipynb.

# %% auto 0
__all__ = ['Sig', 'Param']

# %% ../nbs/09_insp.ipynb 6
import inspect

# %% ../nbs/09_insp.ipynb 8
import types
from typing import (
    TypeAlias, Literal, Union, List, Set,
    Iterable, Tuple, ForwardRef, TypeVar, TypeGuard
)

# %% ../nbs/09_insp.ipynb 10
#| export


# %% ../nbs/09_insp.ipynb 12
#| export


# %% ../nbs/09_insp.ipynb 14
#| export


# %% ../nbs/09_insp.ipynb 16
Sig:   TypeAlias = inspect.Signature
Param: TypeAlias = inspect.Parameter



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_nums.ipynb.

# %% auto 0
__all__ = ['Numeric', 'Numbers', 'NumStr']

# %% ../nbs/02_nums.ipynb 6
from numbers import Number

# %% ../nbs/02_nums.ipynb 8
from typing import (TypeAlias, Union, List,)

# %% ../nbs/02_nums.ipynb 10
#| export


# %% ../nbs/02_nums.ipynb 12
#| export


# %% ../nbs/02_nums.ipynb 14
#| export


# %% ../nbs/02_nums.ipynb 16
Numeric: TypeAlias = Union[int, float, complex, Number]
Numbers: TypeAlias = List[Numeric]
NumStr: TypeAlias = Union[Numeric, str]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/53_biol.ipynb.

# %% auto 0
__all__ = ['Adata', 'Adatas', 'AnnData', 'AnnDatas']

# %% ../nbs/53_biol.ipynb 6
import inspect
from numbers import Number

# %% ../nbs/53_biol.ipynb 8
import types
from typing import (
    Type, TypeAlias, Tuple,
    ClassVar, Any, Optional, Callable, Iterable, ForwardRef
)
from typing import TypeAlias, List, Union, Iterable, List

# %% ../nbs/53_biol.ipynb 11
try:
    import anndata as ad
    adata  = ad.AnnData
    adatas = List[adata]
except ImportError:
    # Define a placeholder type if anndata is not installed
    adata = 'anndata.AnnData'
    adatas = List[adata]

# %% ../nbs/53_biol.ipynb 13
#| export


# %% ../nbs/53_biol.ipynb 15
from .nums import Numbers

# %% ../nbs/53_biol.ipynb 18
Adata: TypeAlias = adata
Adatas: TypeAlias = adatas

AnnData: TypeAlias = adata
AnnDatas: TypeAlias = adatas



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/110_npty.ipynb.

# %% auto 0
__all__ = ['NPArray', 'ndarray', 'NPMatrix', 'ndmatrix', 'DataSource']

# %% ../nbs/110_npty.ipynb 6
#| export


# %% ../nbs/110_npty.ipynb 8
from typing import TypeAlias

# %% ../nbs/110_npty.ipynb 11
try:
    import numpy as np
    arr = np.ndarray
    mtx = np.matrix
    dsr = np.DataSource
    
except ImportError:
    # Define a placeholder type if numpy is not installed
    arr = 'np.ndarray'
    mtx = 'np.matrix'
    dsr = 'np.DataSource'

# %% ../nbs/110_npty.ipynb 13
#| export


# %% ../nbs/110_npty.ipynb 15
#| export


# %% ../nbs/110_npty.ipynb 18
NPArray: TypeAlias = arr
ndarray: TypeAlias = arr

NPMatrix: TypeAlias = mtx
ndmatrix: TypeAlias = mtx

DataSource: TypeAlias = dsr



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/40_iter.ipynb.

# %% auto 0
__all__ = ['IterStr', 'IterNum', 'IterBool']

# %% ../nbs/40_iter.ipynb 6
#| export


# %% ../nbs/40_iter.ipynb 8
from typing import (TypeAlias, Iterable, )

# %% ../nbs/40_iter.ipynb 10
#| export


# %% ../nbs/40_iter.ipynb 12
#| export


# %% ../nbs/40_iter.ipynb 14
from .bool import Boolish
from .nums import Numeric

# %% ../nbs/40_iter.ipynb 16
IterStr: TypeAlias = Iterable[str]
IterNum: TypeAlias = Iterable[Numeric]
IterBool: TypeAlias = Iterable[Boolish]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/04_path.ipynb.

# %% auto 0
__all__ = ['PathType', 'PathLike', 'DirPath', 'DirNames', 'FileNames', 'WalkTuple', 'WalkParts']

# %% ../nbs/04_path.ipynb 6
import os, pathlib

# %% ../nbs/04_path.ipynb 8
from typing import (
    TypeAlias, Literal, Union, List, Set,
    Iterable, Tuple, ForwardRef
)

# %% ../nbs/04_path.ipynb 10
#| export


# %% ../nbs/04_path.ipynb 12
#| export


# %% ../nbs/04_path.ipynb 14
#| export


# %% ../nbs/04_path.ipynb 16
PathType: TypeAlias = Union[pathlib.Path, pathlib.PosixPath]
PathLike: TypeAlias = Union[str, os.PathLike, pathlib.Path, PathType]

# %% ../nbs/04_path.ipynb 18
DirPath:   TypeAlias = PathLike
DirNames:  TypeAlias = List[PathLike]
FileNames: TypeAlias = List[PathLike]
WalkTuple: TypeAlias = Tuple[DirPath, DirNames, FileNames]
WalkParts: TypeAlias = Iterable[WalkTuple]

# %% ../nbs/04_path.ipynb 20
# DirLike: TypeAlias = Union['Dir', 'rdir', 'sdir', 'bdir', 'ndir', 'dsrc', 'dtry']
# BaseLike: TypeAlias = Union['Base', 'base',  'bdir', 'btry', 'bpth', 'bsrc', ]
# Entry: TypeAlias = Union[PathLike, 'Entry', 'BaseEntry', 'StructuredEntry', 'ntry']
# BaseType: TypeAlias   = Union[PathLike, Entry, BaseLike, Entry]
# DirType:  TypeAlias   = Union[PathLike, 'StructuredPath', 'DirEntry',  'StructuredDir', DirLike, 'dpath']
# FileType: TypeAlias   = Union[PathLike, 'StructuredPath', 'FileEntry', 'StructuredFile',  'fpath']
# PathType: TypeAlias   = Union[PathLike, 'StructuredPath', 'PathEntry',  Entry,  BaseType, 'ppath']



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/70_imgs.ipynb.

# %% auto 0
__all__ = ['Img', 'Image', 'ImageLike', 'ImageSize', 'ImgSize', 'Video', 'VideoCapture', 'VidCap', 'Pnt', 'Point', 'BBox',
           'BoundingBox', 'RGB', 'RGBA', 'Color']

# %% ../nbs/70_imgs.ipynb 6
import inspect
from numbers import Number

# %% ../nbs/70_imgs.ipynb 8
from typing import (Union, Tuple, TypeAlias, ForwardRef)

# %% ../nbs/70_imgs.ipynb 11
try:
    from cv2 import VideoCapture as vidcap
except ImportError:
    vidcap = ForwardRef('cv2.VideoCapture', module='cv2', is_class=True)

# %% ../nbs/70_imgs.ipynb 13
try:
    from PIL.Image import Image as image
except ImportError:
    image = ForwardRef('PIL.Image.Image', module='PIL.Image', is_class=True)

# %% ../nbs/70_imgs.ipynb 15
#| export


# %% ../nbs/70_imgs.ipynb 17
from .npty import NPArray
from .nums import Numeric

# %% ../nbs/70_imgs.ipynb 20
Img: TypeAlias = image
Image: TypeAlias = image
ImageLike: TypeAlias = Union[NPArray, Image]

ImageSize: TypeAlias = Tuple[int, int]
ImgSize: TypeAlias = ImageSize

# %% ../nbs/70_imgs.ipynb 22
Video: TypeAlias = vidcap
VideoCapture: TypeAlias = Video
VidCap: TypeAlias = Video

# %% ../nbs/70_imgs.ipynb 24
Pnt: TypeAlias = Tuple[Numeric, Numeric]
Point: TypeAlias = Pnt

BBox: TypeAlias = Tuple[Pnt, Pnt]
BoundingBox: TypeAlias = BBox

# %% ../nbs/70_imgs.ipynb 26
RGB: TypeAlias = Tuple[int, int, int]
RGBA: TypeAlias = Tuple[int, int, int, int]

Color: TypeAlias = Union[RGB, RGBA]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/42_sets.ipynb.

# %% auto 0
__all__ = ['SetStr', 'SetNum', 'SetBool']

# %% ../nbs/42_sets.ipynb 6
import inspect

# %% ../nbs/42_sets.ipynb 8
from typing import (TypeAlias, Set)

# %% ../nbs/42_sets.ipynb 10
#| export


# %% ../nbs/42_sets.ipynb 12
#| export


# %% ../nbs/42_sets.ipynb 14
from .bool import Boolish
from .nums import Numeric

# %% ../nbs/42_sets.ipynb 16
SetStr: TypeAlias = Set[str]
SetNum: TypeAlias = Set[Numeric]
SetBool: TypeAlias = Set[Boolish]
